import 'package:flutter/services.dart';

class PhoneNumberFormatter extends TextInputFormatter {
  final String _exitCode = '+';
  late final bool _usingExitCode;
  final String _zero = '0';

  PhoneNumberFormatter({final bool usingExitCode = false}) {
    _usingExitCode = usingExitCode;
  }

  @override
  TextEditingValue formatEditUpdate(
      final TextEditingValue oldValue, final TextEditingValue newValue) {
    String value = newValue.text.replaceAll(_exitCode, '');
    bool startsWithZero = false;
    while (value.startsWith(_zero)) {
      startsWithZero = true;
      value = value.replaceFirst(_zero, '');
    }
    if (_usingExitCode) {
      value = '$_exitCode$value';
    } else if (startsWithZero) {
      value = '$_zero$value';
    }
    return TextEditingValue(
        selection: TextSelection.collapsed(offset: value.length), text: value);
  }
}
