import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_custom_form_component/widget/const.dart';
import 'package:flutter_custom_form_component/widget/field/box_decorations.dart';
import 'package:flutter_custom_form_component/widget/field/mixin.dart';
import 'package:flutter_custom_form_component/widget/field/one_value/mixin.dart';
import 'package:flutter_custom_form_component/widget/field/one_value/widget.dart';
import 'package:flutter_custom_form_component/widget/field/phone_number/formatter.dart';
import 'package:flutter_custom_form_component/widget/field/text_styles.dart';

class PhoneNumberFieldWidget extends OneValueFieldWidget {
  late final bool _usingExitCode;

  PhoneNumberFieldWidget(
      {final TextInputAction action = fieldAction,
      final BoxDecorations? decorations,
      final bool enabled = fieldEnabled,
      final String? hint,
      final double horizontalPadding = fieldHorizontalPadding,
      final String? initialValue,
      final Key? key,
      final Function(dynamic value)? onChanged,
      final TextStyles? styles,
      final bool usingExitCode = false})
      : super(
            action: action,
            decorations: decorations,
            enabled: enabled,
            hint: hint,
            horizontalPadding: horizontalPadding,
            initialValue: initialValue,
            key: key,
            onChanged: onChanged,
            styles: styles) {
    _usingExitCode = usingExitCode;
  }

  @override
  State<StatefulWidget> createState() => _State();
}

class _State extends State<PhoneNumberFieldWidget>
    with FieldMixin, OneValueFieldMixin {
  final FocusNode _focusNode = FocusNode();
  late final PhoneNumberFormatter _formatter;

  @override
  Widget build(final BuildContext context) => container(TextField(
      controller: controller,
      decoration: InputDecoration(
          border: InputBorder.none,
          hintStyle: widget.styles.hint,
          hintText: widget.hint),
      enabled: widget.enabled,
      focusNode: _focusNode,
      inputFormatters: [
        FilteringTextInputFormatter.allow(RegExp('[0-9+]')),
        _formatter
      ],
      keyboardType: TextInputType.phone,
      style: style,
      textInputAction: widget.action));

  @override
  void dispose() {
    _focusNode.dispose();
    disposeMixin();
    super.dispose();
  }

  @override
  void initState() {
    super.initState();
    _formatter = PhoneNumberFormatter(usingExitCode: widget._usingExitCode);
    initStateMixin(
        initialValue: widget.initialValue, onChanged: widget.onChanged);
  }

  @override
  BoxDecoration get decoration {
    if (_focusNode.hasFocus) {
      return widget.decorations.active;
    }
    if (valid) {
      return widget.decorations.box;
    }
    return widget.decorations.error;
  }

  @override
  double get horizontalPadding => widget.horizontalPadding;

  @override
  TextStyle get style {
    if (_focusNode.hasFocus) {
      return widget.styles.active;
    }
    if (valid) {
      return widget.styles.text;
    }
    return widget.styles.error;
  }

  @override
  String get value => super.value;
}
